package com.tencent.flutter_integration;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.io.Serializable;
import java.lang.ref.SoftReference;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;

import androidx.annotation.NonNull;
import io.flutter.embedding.android.FlutterFragment;
import io.flutter.embedding.android.FlutterView;
import io.flutter.embedding.android.RenderMode;
import io.flutter.embedding.android.TransparencyMode;
import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.renderer.FlutterRenderer;
import io.flutter.embedding.engine.renderer.FlutterUiDisplayListener;

public class FlutterIntegrationFragment extends FlutterFragment {

    public String routeName;
    public Serializable params;

    private static final String KEY_ROUTE_NAME = "FlutterIntegrationFragment_KEY_ROUTE_NAME";
    private static final String KEY_PARAMS = "FlutterIntegrationFragment_KEY_PARAMS";
    private static final String KEY_PAGEID = "FlutterIntegrationFragment_KEY_PAGEID";

    private String mPageId;
    private HookFlutterView mFlutterView;
    private boolean mFirst = true;
    private boolean mForceOpenPage = true;

    public FlutterIntegrationFragment() {
        super();
    }

    public static FlutterIntegrationFragment newInstance(String routeName) {
        return newInstance(routeName, null);
    }

    public static FlutterIntegrationFragment newInstance(String routeName, Serializable params) {
        FlutterIntegrationFragment fragment = new NewEngineFragmentBuilder(FlutterIntegrationFragment.class)
                .initialRoute("/")
                .transparencyMode(TransparencyMode.opaque)
                .renderMode(RenderMode.texture)
                .build();
        fragment.routeName = routeName;
        fragment.params = params;

        return fragment;
    }

    @Override
    public FlutterEngine provideFlutterEngine(Context context) {
        return FlutterIntegrationManager.sharedInstance().container.flutterEngine;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        Log.e("FlutterIntegration", String.format("onCreateView, pageId:%s", mPageId));
        View splashView = super.onCreateView(inflater, container, savedInstanceState);

        // 清除旧的listener
        if (splashView instanceof ViewGroup) {
            ViewGroup sv = (ViewGroup) splashView;
            if (sv.getChildCount() > 0 && sv.getChildAt(0) instanceof FlutterView) {
                FlutterView fv = (FlutterView) sv.getChildAt(0);

                try {
                    Class<?> clz = FlutterView.class;
                    Field field = clz.getDeclaredField("flutterUiDisplayListener");
                    field.setAccessible(true);
                    FlutterUiDisplayListener listener = (FlutterUiDisplayListener) field.get(fv);
                    FlutterRenderer flutterRenderer = fv.getAttachedFlutterEngine().getRenderer();
                    flutterRenderer.removeIsDisplayingFlutterUiListener(listener);

                } catch (Exception e) {
                    Log.w("hook FlutterView failed", e);
                }
            }
        }

        FlutterView.RenderMode oldRenderMode;
        FlutterView.TransparencyMode oldTransparencyMode;
        if(getRenderMode() == RenderMode.surface){
            oldRenderMode = FlutterView.RenderMode.surface;
        }else{
            oldRenderMode = FlutterView.RenderMode.texture;
        }
        if(getTransparencyMode() == TransparencyMode.opaque){
            oldTransparencyMode = FlutterView.TransparencyMode.opaque;
        }else{
            oldTransparencyMode = FlutterView.TransparencyMode.transparent;
        }

        mFlutterView = new HookFlutterView(getActivity(), oldRenderMode, oldTransparencyMode);

        // hook FlutterView attach&detach
        try {
            Class<?> clz = FlutterFragment.class;
            Field field = clz.getDeclaredField("delegate");
            field.setAccessible(true);
            Object delegate = field.get(this);
            clz = Class.forName("io.flutter.embedding.android.FlutterActivityAndFragmentDelegate");
            field = clz.getDeclaredField("flutterUiDisplayListener");
            field.setAccessible(true);
            Field fieldFlutterView = clz.getDeclaredField("flutterView");
            fieldFlutterView.setAccessible(true);
            fieldFlutterView.set(delegate, mFlutterView);
            FlutterUiDisplayListener flutterUiDisplayListener = (FlutterUiDisplayListener) field.get(delegate);
            mFlutterView.addOnFirstFrameRenderedListener(flutterUiDisplayListener);

            clz = Class.forName("io.flutter.embedding.android.FlutterSplashView");
            Class<?> screenClz = Class.forName("io.flutter.embedding.android.SplashScreen");
            Method m = clz.getMethod("displayFlutterViewWithSplash", new Class[]{FlutterView.class, screenClz});
            m.invoke(splashView, mFlutterView, provideSplashScreen());

        } catch (Exception e) {
            Log.w("hook FlutterView failed", e);
        }

        return splashView;
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        Log.e("FlutterIntegration", String.format("onAttach, pageId:%s", mPageId));
    }

    @Override
    public void onStart() {
        super.onStart();
        Log.e("FlutterIntegration", String.format("onStart, pageId:%s", mPageId));
    }

    public void forceResume() {
        mForceOpenPage = true;
        onResume();
    }

    @Override
    public void onResume() {
        super.onResume();

        // 只有第一次启动或者确定需要openPage的场景才进行操作
        if (mFirst || mForceOpenPage) {
            FlutterIntegrationManager.sharedInstance().container.fragment = FlutterIntegrationFragment.this;
            mFlutterView.hookAttachToFlutterEngine(FlutterIntegrationManager.sharedInstance().container.flutterEngine);
            openPage();
            mFirst = false;
            mForceOpenPage = false;
            FlutterIntegrationManager.sharedInstance().container.flutterView = new SoftReference<FlutterView>(mFlutterView);
        }
        Log.e("FlutterIntegration", String.format("onResume, pageId:%s", mPageId));
    }

    @Override
    public void onPause() {
        super.onPause();
        Log.e("FlutterIntegration", String.format("onPause, pageId:%s", mPageId));
    }

    @Override
    public void onStop() {
        super.onStop();
        mFlutterView.detachFromFlutterEngine();

        // 如果是当前页面拿着引擎，置空避免无法释放，否则尝试恢复，避免onStop在onResume之后
        if (FlutterIntegrationManager.sharedInstance().container.fragment != null &&
                FlutterIntegrationManager.sharedInstance().container.fragment.mPageId.equals(this.mPageId)) {
            Log.e("FlutterIntegration", String.format("onStop, pageId:%s, hit", mPageId));
            FlutterIntegrationManager.sharedInstance().container.fragment = null;

        } else if (FlutterIntegrationManager.sharedInstance().container.fragment != null) {
            Log.e("FlutterIntegration", String.format("onStop, pageId:%s, recover", mPageId));
            FlutterIntegrationManager.sharedInstance().container.fragment.forceResume();
        }
        Log.e("FlutterIntegration", String.format("onStop, pageId:%s", mPageId));
    }

    @Override
    public void onDetach() {
        super.onDetach();

        // 如果是当前页面拿着引擎，置空避免无法释放，否则尝试恢复，避免onDetach在onResume之后
        if (FlutterIntegrationManager.sharedInstance().container.fragment != null &&
                FlutterIntegrationManager.sharedInstance().container.fragment.mPageId.equals(this.mPageId)) {
            Log.e("FlutterIntegration", String.format("onDetach, pageId:%s, hit", mPageId));
            FlutterIntegrationManager.sharedInstance().container.fragment = null;

        } else if (FlutterIntegrationManager.sharedInstance().container.fragment != null) {
            Log.e("FlutterIntegration", String.format("onDetach, pageId:%s, recover", mPageId));
            FlutterIntegrationManager.sharedInstance().container.fragment.forceResume();
        }
        Log.e("FlutterIntegration", String.format("onDetach, pageId:%s", mPageId));
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        removePage();
        Log.e("FlutterIntegration", String.format("onDestroyView, pageId:%s", mPageId));
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putString(KEY_ROUTE_NAME, routeName);
        outState.putSerializable(KEY_PARAMS, params);
        outState.putString(KEY_PAGEID, mPageId);
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        if (savedInstanceState != null) {
            routeName = savedInstanceState.getString(KEY_ROUTE_NAME);
            mPageId = savedInstanceState.getString(KEY_PAGEID);
            params = savedInstanceState.getSerializable(KEY_PARAMS);
            mFirst = true;
        }
    }

    @Override
    public boolean shouldDestroyEngineWithHost() {
        return false;
    }

    @Override
    public boolean shouldAttachEngineToActivity() {
        return false;
    }

    @SuppressLint("DefaultLocale")
    private String getPageId() {
        if (mPageId == null || mPageId.length() <= 0) {
            mPageId = String.format("%s_%d", routeName, System.currentTimeMillis());
        }
        return mPageId;
    }

    private void openPage() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("routeName", routeName);
        map.put("pageId", getPageId());
        if (params != null) {
            map.put("params", params);
        }

        FlutterIntegrationManager.sharedInstance().container.methodChannel.invokeMethod("openPage", map);
        Log.e("FlutterIntegration", String.format("openPage, pageId:%s", mPageId));
    }

    private void removePage() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("pageId", mPageId);

        FlutterIntegrationManager.sharedInstance().container.methodChannel.invokeMethod("removePage", map);
        Log.e("FlutterIntegration", String.format("removePage, pageId:%s", mPageId));
    }

    @NonNull
    @Override
    public String toString() {
        return super.toString() + ", pageId:" + getPageId();
    }
}
