package com.tencent.flutter_integration;

import android.content.Context;
import android.util.Log;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.FlutterJNI;
import io.flutter.embedding.engine.dart.DartExecutor;
import io.flutter.embedding.engine.renderer.FlutterRenderer;
import io.flutter.embedding.engine.renderer.FlutterUiDisplayListener;
import io.flutter.embedding.engine.systemchannels.LifecycleChannel;

public class FlutterIntegrationManager {

    private static volatile FlutterIntegrationManager mManager;

    public FlutterIntegrationEngineContainer container = new FlutterIntegrationEngineContainer();

    public static FlutterIntegrationManager sharedInstance() {
        if (mManager == null) {
            synchronized (FlutterIntegrationManager.class) {
                if (mManager == null) {
                    mManager = new FlutterIntegrationManager();
                }
            }
        }
        return mManager;
    }

    public void initEngine(Context context) {
        if (container.flutterEngine == null) {
            FlutterEngine flutterEngine = new FlutterEngine(context);
            container.flutterEngine = flutterEngine;
            flutterEngine.getDartExecutor().executeDartEntrypoint(
                    DartExecutor.DartEntrypoint.createDefault()
            );
            disableDelegate();

            try {
                Class clz = Class.forName("io.flutter.plugins.GeneratedPluginRegistrant");
                Method method = clz.getDeclaredMethod("registerWith", FlutterEngine.class);
                method.invoke(null, flutterEngine);
            } catch (Throwable t) {
                Log.i("flutter_integration", t.toString());
            }

        }
    }

    private void disableDelegate() {
        try {
            Class<?> clz = FlutterEngine.class;
            Field field = clz.getDeclaredField("lifecycleChannel");
            field.setAccessible(true);
            LifecycleChannel channel = (LifecycleChannel) field.get(FlutterIntegrationManager.sharedInstance().container.flutterEngine);
            HookLifecycleChannel hookLifecycleChannel = new HookLifecycleChannel(channel);
            field.set(container.flutterEngine, hookLifecycleChannel);

        } catch (Exception e) {
            Log.w("hook channel failed!", e);
        }

        try {
            Class<?> clz = FlutterEngine.class;
            Field field = clz.getDeclaredField("flutterJNI");
            field.setAccessible(true);
            FlutterJNI flutterJNI = (FlutterJNI) field.get(container.flutterEngine);

            HookFlutterRenderer proxy = new HookFlutterRenderer(flutterJNI);
            field = clz.getDeclaredField("renderer");
            field.setAccessible(true);
            field.set(container.flutterEngine, proxy);
        } catch (Exception e) {
            Log.w("hook renderer failed!", e);
        }
    }

    static class HookLifecycleChannel extends LifecycleChannel {

        private LifecycleChannel mChannel;

        HookLifecycleChannel(LifecycleChannel channel) {
            super(new DartExecutor(null, null));
            mChannel = channel;
        }

        public void appIsInactive() {
            Log.v("HookLifecycleChannel", "Sending AppLifecycleState.inactive message.");
            mChannel.channel.send("AppLifecycleState.inactive");
        }

        public void appIsResumed() {
            Log.v("HookLifecycleChannel", "Sending AppLifecycleState.resumed message.");
            mChannel.channel.send("AppLifecycleState.resumed");
        }

        public void appIsPaused() {
            Log.v("HookLifecycleChannel", "Sending AppLifecycleState.paused message.");
//            mChannel.send("AppLifecycleState.paused");
        }

        public void appIsDetached() {
            Log.v("HookLifecycleChannel", "Sending AppLifecycleState.detached message.");
//            mChannel.send("AppLifecycleState.detached");
        }
    }

    public static class HookFlutterRenderer extends FlutterRenderer {

        HookFlutterRenderer(FlutterJNI subject) {
            super(subject);
        }

        @Override
        public void stopRenderingToSurface() {
            Log.i("HookFlutterRenderer", "stopRenderingToSurface");
//            super.stopRenderingToSurface();
        }
        
        @Override
        public void setViewportMetrics(ViewportMetrics viewportMetrics) {
            if (viewportMetrics.height == 0 && viewportMetrics.width == 0) {
                return;
            }
            super.setViewportMetrics(viewportMetrics);
        }

    }

}
