package com.tencent.flutter_integration;

import android.content.Context;
import android.util.AttributeSet;
import android.util.Log;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import io.flutter.embedding.android.FlutterView;
import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.renderer.FlutterRenderer;
import io.flutter.embedding.engine.renderer.FlutterUiDisplayListener;

public class HookFlutterView extends FlutterView {

    private boolean detached = false;

    public HookFlutterView(@NonNull Context context) {
        super(context);
    }

    public HookFlutterView(@NonNull Context context, @NonNull RenderMode renderMode) {
        super(context, renderMode);
    }

    public HookFlutterView(@NonNull Context context, @NonNull TransparencyMode transparencyMode) {
        super(context, RenderMode.surface, transparencyMode);
    }

    public HookFlutterView(@NonNull Context context, @NonNull RenderMode renderMode, @NonNull TransparencyMode transparencyMode) {
        super(context, renderMode, transparencyMode);
    }

    public HookFlutterView(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    @Override
    public void detachFromFlutterEngine() {
        detached = true;

        try {
            Class<?> clz = FlutterView.class;
            Field field = clz.getDeclaredField("flutterUiDisplayListener");
            field.setAccessible(true);
            FlutterUiDisplayListener listener = (FlutterUiDisplayListener) field.get(this);
            FlutterRenderer flutterRenderer = getAttachedFlutterEngine().getRenderer();
            flutterRenderer.removeIsDisplayingFlutterUiListener(listener);

        } catch (Exception e) {
            Log.w("hook FlutterView failed", e);
        }
//        super.detachFromFlutterEngine();
    }

    @Override
    public boolean isAttachedToFlutterEngine() {
        if (FlutterIntegrationManager.sharedInstance().container.flutterView != null &&
                FlutterIntegrationManager.sharedInstance().container.flutterView.get() == this) {
            return true;
        }
        if (detached) {
            return false;
        }
        return super.isAttachedToFlutterEngine();
    }

    @Override
    public void attachToFlutterEngine(FlutterEngine flutterEngine) {
        Log.e("FlutterIntegration", "attachToFlutterEngine :" + this.toString());
//        super.attachToFlutterEngine(flutterEngine);
    }

    public void hookAttachToFlutterEngine(FlutterEngine flutterEngine) {
        Log.e("FlutterIntegration", "hookAttachToFlutterEngine :" + this.toString());
        super.attachToFlutterEngine(flutterEngine);
        detached = false;
        try {
            Class<?> clz = FlutterView.class;
            Method m = clz.getDeclaredMethod("sendViewportMetricsToFlutter");
            m.setAccessible(true);
            m.invoke(this);

        } catch (Exception e) {
            Log.w("hook FlutterView fail!", e);
        }
    }

}
